@@ -49,6 +49,8 @@ class PortalBase:
                             on-board NeoPixel. Defaults to ``None``, to not use the status LED
     :param json_transform: A function or a list of functions to call with the parsed JSON.
                         Changes and additions are permitted for the ``dict`` object.
+    :param success_callback: A function we'll call if you like, when we fetch data successfully.
+                             Defaults to ``None``.
     :param debug: Turn on debug print outs. Defaults to False.
 
     """
@@ -64,9 +66,9 @@ class PortalBase:
         json_path=None,
         regexp_path=None,
         json_transform=None,
+        success_callback=None,
         debug=False,
     ):
-
         self.network = network
         self.graphics = graphics
         self.splash = self.graphics.splash
@@ -82,15 +84,14 @@ class PortalBase:
             self._alarm = alarm
         except ImportError:
             self._alarm = None
-
         self._debug = debug
-        self._url = None
         self.url = url
         self._headers = headers
         self._json_path = None
         self.json_path = json_path
 
         self._regexp_path = regexp_path
+        self._success_callback = success_callback
 
         # Add any JSON translators
         if json_transform:
@@ -165,7 +166,7 @@ class PortalBase:
         :param text_transform: A function that will be called on the text before display
         :param int text_scale: The factor to scale the default size of the text by
         :param float line_spacing: The factor to space the lines apart
-        :param (float, float) text_anchor_point: Values between 0 and 1 to indicate where the text
+        :param (float,float) text_anchor_point: Values between 0 and 1 to indicate where the text
                                                  position is relative to the label
         :param bool is_data: If True, fetch will attempt to update the label
         """
@@ -354,17 +355,28 @@ class PortalBase:
 
         """
         if refresh_url:
-            self._url = refresh_url
+            self.url = refresh_url
         values = []
 
         values = self.network.fetch_data(
-            self._url,
+            self.url,
             headers=self._headers,
             json_path=self._json_path,
             regexp_path=self._regexp_path,
             timeout=timeout,
         )
 
+        # if we have a callback registered, call it now
+        if self._success_callback:
+            self._success_callback(values)
+
+        self._fill_text_labels(values)
+
+        if len(values) == 1:
+            return values[0]
+        return values
+
+    def _fill_text_labels(self, values):
         # fill out all the text blocks
         if self._text:
             value_index = 0  # In case values and text is not the same
@@ -382,9 +394,6 @@ class PortalBase:
                         string = values[value_index]  # ok it's a string
                 self._fetch_set_text(string, index=i)
                 value_index += 1
-        if len(values) == 1:
-            return values[0]
-        return values
 
     def get_local_time(self, location=None):
         """Accessor function for get_local_time()"""
@@ -426,21 +435,6 @@ class PortalBase:
         """
         return self.network.get_io_group(group_key)
 
-    @property
-    def url(self):
-        """
-        Get or set the URL of your data source.
-        """
-        return self._url
-
-    @url.setter
-    def url(self, value):
-        self._url = value
-        if value and not self.network.uselocal:
-            self.network.connect()
-            # if self._debug:
-            #    print("My IP address is", self.network.ip_address)
-
     @property
     def json_path(self):
         """
