@@ -26,10 +26,17 @@ import time
 import gc
 from micropython import const
 from adafruit_io.adafruit_io import IO_HTTP, AdafruitIO_RequestError
-import supervisor
-import rtc
 from adafruit_fakerequests import Fake_Requests
 
+try:
+    import supervisor
+except ImportError:
+    supervisor = None
+
+try:
+    import rtc
+except ImportError:
+    rtc = None
 
 try:
     from secrets import secrets
@@ -38,7 +45,6 @@ except ImportError:
         """WiFi settings are kept in secrets.py, please add them there!
 the secrets dictionary must contain 'ssid' and 'password' at a minimum"""
     )
-    raise
 
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_PortalBase.git"
@@ -80,21 +86,28 @@ class NetworkBase:
     :param bool extract_values: If true, single-length fetched values are automatically extracted
                                 from lists and tuples. Defaults to ``True``.
     :param debug: Turn on debug print outs. Defaults to False.
+    :param list secrets_data: An optional list in place of the data contained in the secrets.py file
 
     """
 
     # pylint: disable=too-many-instance-attributes, too-many-locals, too-many-branches, too-many-statements
     def __init__(
-        self,
-        wifi_module,
-        *,
-        extract_values=True,
-        debug=False,
+        self, wifi_module, *, extract_values=True, debug=False, secrets_data=None
     ):
         self._wifi = wifi_module
         self._debug = debug
         self.json_transform = []
         self._extract_values = extract_values
+        self._json_types = [
+            "application/json",
+            "application/javascript",
+            "application/geo+json",
+        ]
+
+        if secrets_data is not None:
+            self._secrets = secrets_data
+        else:
+            self._secrets = secrets
 
         # This may be removed. Using for testing
         self.requests = None
@@ -162,14 +175,15 @@ class NetworkBase:
         self.connect()
         api_url = None
         try:
-            aio_username = secrets["aio_username"]
-            aio_key = secrets["aio_key"]
+            aio_username = self._secrets["aio_username"]
+            aio_key = self._secrets["aio_key"]
         except KeyError:
             raise KeyError(
                 "\n\nOur time service requires a login/password to rate-limit. Please register for a free adafruit.io account and place the user/key in your secrets file under 'aio_username' and 'aio_key'"  # pylint: disable=line-too-long
             ) from KeyError
 
-        location = secrets.get("timezone", location)
+        if location is None:
+            location = self._secrets.get("timezone", location)
         if location:
             print("Getting time for timezone", location)
             api_url = (TIME_SERVICE + "&tz=%s") % (aio_username, aio_key, location)
@@ -184,7 +198,7 @@ class NetworkBase:
                     "Error connection to Adafruit IO. The response was: "
                     + response.text
                 )
-                raise ValueError(error_message)
+                raise RuntimeError(error_message)
             if self._debug:
                 print("Time request: ", api_url)
                 print("Time reply: ", response.text)
@@ -204,7 +218,8 @@ class NetworkBase:
         now = time.struct_time(
             (year, month, mday, hours, minutes, seconds, week_day, year_day, is_dst)
         )
-        rtc.RTC().datetime = now
+        if rtc is not None:
+            rtc.RTC().datetime = now
 
         # now clean up
         response.close()
@@ -286,8 +301,11 @@ class NetworkBase:
         self._wifi.neo_status(STATUS_CONNECTING)
         while not self._wifi.is_connected:
             # secrets dictionary must contain 'ssid' and 'password' at a minimum
-            print("Connecting to AP", secrets["ssid"])
-            if secrets["ssid"] == "CHANGE ME" or secrets["password"] == "CHANGE ME":
+            print("Connecting to AP", self._secrets["ssid"])
+            if (
+                self._secrets["ssid"] == "CHANGE ME"
+                or self._secrets["password"] == "CHANGE ME"
+            ):
                 change_me = "\n" + "*" * 45
                 change_me += "\nPlease update the 'secrets.py' file on your\n"
                 change_me += "CIRCUITPY drive to include your local WiFi\n"
@@ -297,7 +315,7 @@ class NetworkBase:
                 raise OSError(change_me)
             self._wifi.neo_status(STATUS_NO_CONNECTION)  # red = not connected
             try:
-                self._wifi.connect(secrets["ssid"], secrets["password"])
+                self._wifi.connect(self._secrets["ssid"], self._secrets["password"])
                 self.requests = self._wifi.requests
             except RuntimeError as error:
                 print("Could not connect to internet", error)
@@ -308,8 +326,8 @@ class NetworkBase:
         self.connect()
 
         try:
-            aio_username = secrets["aio_username"]
-            aio_key = secrets["aio_key"]
+            aio_username = self._secrets["aio_username"]
+            aio_key = self._secrets["aio_key"]
         except KeyError:
             raise KeyError(
                 "Adafruit IO secrets are kept in secrets.py, please add them there!\n\n"
@@ -407,8 +425,6 @@ class NetworkBase:
 
         """
         gc.collect()
-        if self._debug:
-            print("Free mem: ", gc.mem_free())  # pylint: disable=no-member
 
         response = None
         if self.uselocal:
@@ -426,6 +442,63 @@ class NetworkBase:
 
         return response
 
+    def add_json_content_type(self, content_type):
+        """
+        Add a JSON content type
+
+        :param str type: The content JSON type like 'application/json'
+
+        """
+        if isinstance(content_type, str):
+            self._json_types.append(content_type)
+
+    def _detect_content_type(self, headers):
+        if "content-type" in headers:
+            if "image/" in headers["content-type"]:
+                return CONTENT_IMAGE
+            for json_type in self._json_types:
+                if json_type in headers["content-type"]:
+                    return CONTENT_JSON
+        return CONTENT_TEXT
+
+    def check_response(self, response):
+        """
+        Check the response object status code, change the lights, and return content type
+
+        :param response: The response object from a network call
+
+        """
+        headers = self._get_headers(response)
+
+        if self._debug:
+            print("Headers:", headers)
+        if response.status_code == 200:
+            print("Reply is OK!")
+            self.neo_status(STATUS_DATA_RECEIVED)  # green = got data
+            content_type = self._detect_content_type(headers)
+        else:
+            if self._debug:
+                if "content-length" in headers:
+                    print("Content-Length: {}".format(int(headers["content-length"])))
+                if "date" in headers:
+                    print("Date: {}".format(headers["date"]))
+            self.neo_status((100, 0, 0))  # red = http error
+            raise HttpError(
+                "Code {}: {}".format(
+                    response.status_code, response.reason.decode("utf-8")
+                )
+            )
+
+        return content_type
+
+    @staticmethod
+    def _get_headers(response):
+        headers = {}
+        for title, content in response.headers.items():
+            headers[title.lower()] = content
+        gc.collect()
+        return headers
+
     def fetch_data(
         self,
         url,
@@ -440,51 +513,32 @@ class NetworkBase:
         :param str url: The URL to fetch from.
         :param list headers: Extra headers to include in the request.
         :param json_path: The path to drill down into the JSON data.
-        :param regexp_path: The path formatted as a regular expression to drill down
-                            into the JSON data.
+        :param regexp_path: The path formatted as a regular expression to search
+                            the text data.
         :param int timeout: The timeout period in seconds.
 
         """
-        json_out = None
-        values = []
-        content_type = CONTENT_TEXT
-
         response = self.fetch(url, headers=headers, timeout=timeout)
+        return self._parse_data(response, json_path=json_path, regexp_path=regexp_path)
 
-        headers = {}
-        for title, content in response.headers.items():
-            headers[title.lower()] = content
-        gc.collect()
-        if self._debug:
-            print("Headers:", headers)
-        if response.status_code == 200:
-            print("Reply is OK!")
-            self.neo_status(STATUS_DATA_RECEIVED)  # green = got data
-            if "content-type" in headers:
-                if "image/" in headers["content-type"]:
-                    content_type = CONTENT_IMAGE
-                elif "application/json" in headers["content-type"]:
-                    content_type = CONTENT_JSON
-                elif "application/javascript" in headers["content-type"]:
-                    content_type = CONTENT_JSON
-        else:
-            if self._debug:
-                if "content-length" in headers:
-                    print("Content-Length: {}".format(int(headers["content-length"])))
-                if "date" in headers:
-                    print("Date: {}".format(headers["date"]))
-            self.neo_status((100, 0, 0))  # red = http error
-            raise HttpError(
-                "Code {}: {}".format(
-                    response.status_code, response.reason.decode("utf-8")
-                )
-            )
+    def _parse_data(
+        self,
+        response,
+        *,
+        json_path=None,
+        regexp_path=None,
+    ):
 
-        if content_type == CONTENT_JSON and json_path is not None:
-            if isinstance(json_path, (list, tuple)) and (
-                not json_path or not isinstance(json_path[0], (list, tuple))
-            ):
-                json_path = (json_path,)
+        json_out = None
+        content_type = self.check_response(response)
+
+        if content_type == CONTENT_JSON:
+            if json_path is not None:
+                # Drill down to the json path and set json_out as that node
+                if isinstance(json_path, (list, tuple)) and (
+                    not json_path or not isinstance(json_path[0], (list, tuple))
+                ):
+                    json_path = (json_path,)
             try:
                 gc.collect()
                 json_out = response.json()
@@ -495,45 +549,75 @@ class NetworkBase:
                 print("Couldn't parse json: ", response.text)
                 raise
             except MemoryError:
-                supervisor.reload()
+                if supervisor is not None:
+                    supervisor.reload()
+                raise
 
+        if content_type == CONTENT_JSON:
+            values = self.process_json(json_out, json_path)
+        elif content_type == CONTENT_TEXT:
+            values = self.process_text(response.text, regexp_path)
+
+        # Clean up
+        json_out = None
+        response = None
+        if self._extract_values and len(values) == 1:
+            values = values[0]
+
+        gc.collect()
+
+        return values
+
+    @staticmethod
+    def process_text(text, regexp_path):
+        """
+        Process text content
+
+        :param str text: The entire text content
+        :param regexp_path: The path formatted as a regular expression to search
+                            the text data.
+
+        """
+        values = []
         if regexp_path:
             import re  # pylint: disable=import-outside-toplevel
 
+            for regexp in regexp_path:
+                values.append(re.search(regexp, text).group(1))
+        else:
+            values = text
+        return values
+
+    def process_json(self, json_data, json_path):
+        """
+        Process JSON content
+
+        :param dict json_data: The JSON data as a dict
+        :param json_path: The path to drill down into the JSON data.
+
+        """
+        values = []
+
         # optional JSON post processing, apply any transformations
         # these MAY change/add element
         for idx, json_transform in enumerate(self.json_transform):
             try:
-                json_transform(json_out)
+                json_transform(json_data)
             except Exception as error:
                 print("Exception from json_transform: ", idx, error)
                 raise
 
         # extract desired text/values from json
-        if json_out is not None and json_path:
+        if json_data is not None and json_path:
             for path in json_path:
                 try:
-                    values.append(self.json_traverse(json_out, path))
+                    values.append(self.json_traverse(json_data, path))
                 except KeyError:
-                    print(json_out)
+                    print(json_data)
                     raise
-        elif content_type == CONTENT_TEXT and regexp_path:
-            for regexp in regexp_path:
-                values.append(re.search(regexp, response.text).group(1))
         else:
-            if json_out:
-                # No path given, so return JSON as string for compatibility
-                import json  # pylint: disable=import-outside-toplevel
-
-                values = json.dumps(response.json())
-            else:
-                values = response.text
-
-        # we're done with the requests object, lets delete it so we can do more!
-        json_out = None
-        response = None
-        gc.collect()
-        if self._extract_values and len(values) == 1:
-            return values[0]
+            # No path given, so return JSON as string for compatibility
+            import json  # pylint: disable=import-outside-toplevel
 
+            values = json.dumps(json_data)
         return values
